# reads list of filenames from input (one filename per line
# as generated by find). Each file should be call-reference
# file as generated by devik's gcc extension.
# Then create dot script describing directed call-graph. The
# functions coveres are all ones from those filenames beginning
# by + sign and extending over all calers and calees from other
# (listed) files too.
# This extending is done by transitive propagation thru call
# graph. You can affect no of displayed functions by tweaking
# $propdepth

$propdepth=2

print "digraph linux {\n";
print "node [ fontname=Helvetica,fontsize=8 ];\n";

# read all input files (names goes from stdin)
while (<>) {
	die if (!/^(\+?)(.*)$/); $e = $1;
	print STDERR "opening $2\n";
	open (F,"$2") || die;

	while (<F>) {
		if (/^F (.+){([^\/]+):(.+)}/) {
			$F{$1}="$2:$3";
			$M{$1}=3 if ($e eq '+');
		} elsif (/^C (.+){(.+):(.+)}\s+(.+)/) {
			$C{"$1:$4"}="$2:$3"
		} 
	}
	close F;
}
# transitive propagation of selected functions
for($i=0;$i<$propdepth;$i++) {
	foreach (keys %C) {
		next if (!/^(.+):(.+)$/);
		$M{$2} |= 1 if ($M{$1} & 1);
		$M{$1} |= 2 if ($M{$2} & 2);
	}
}
# print result
foreach (keys %C) {
	next if (!/^(.+):(.+)$/); $f1=$1; $f2=$2;
	next if (not exists $F{$f2}); # ignore call of unknown fn
	next if ($F{$f2} =~ /\//);    # ignore fns defined in headers
	next if (!($M{$f1} > 0));     # ignore not flagged caller
	print "$f1 -> $f2";
#	print " [ style=bold ]" if ($F{"$f1-$f2"}>1);
	print ";\n";
}
print "}\n";
